/*
 * Copyright 2005 by Lars Torunski
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.torunski.crawler;

import java.util.*;

import edu.emory.mathcs.backport.java.util.concurrent.LinkedBlockingQueue;
import edu.emory.mathcs.backport.java.util.concurrent.ThreadPoolExecutor;
import edu.emory.mathcs.backport.java.util.concurrent.TimeUnit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.torunski.crawler.core.AbstractCrawler;
import com.torunski.crawler.link.Link;
import com.torunski.crawler.parser.PageData;
import com.torunski.crawler.util.StopWatch;

/**
 * Project: Smart & Simple Web Crawler
 * 
 * Crawls through the web with a lot of threads.
 * 
 * @author Lars Torunski
 * @version $Revision: 1.5 $
 */
public class MultiThreadedCrawler extends AbstractCrawler {
    
    private static transient final Log log = LogFactory.getLog(MultiThreadedCrawler.class);
    
    private final Object UPDATE_LOCK = new Object();
    
	private ThreadPoolExecutor loadService;
	private ThreadPoolExecutor parseService;
	
	private int threadsToBeFinished = 0;

	/**
	 * Default constructor for 5 loading threads and 2 parsing threads.
	 */
	public MultiThreadedCrawler() {
		this(5, 2);
	}

	/**
	 * Creates a multi threaded crawler which delegates the load and parse tasks to different threads.
	 * Per RFC 2616 sec 8.1.4 the maximum number of connections allowed per host is 2.
	 * @see org.apache.commons.httpclient.MultiThreadedHttpConnectionManager#DEFAULT_MAX_TOTAL_CONNECTIONS
	 * @param maxLoadThreads maximum of threads for loading the content
	 * @param maxParseThreads maximum of threads for parsing the download
	 */
	public MultiThreadedCrawler(int maxLoadThreads, int maxParseThreads) {
		loadService = new ThreadPoolExecutor(maxLoadThreads, maxLoadThreads, 0L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue());
		parseService = new ThreadPoolExecutor(maxParseThreads, maxParseThreads, 0L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue());
	}

    /**
     * Starts the crawling process in a multi threaded enviroment.
	 */
    public void start(String server, String start) {
        
        // set the default parser
        if (parser == null) {
        	log.debug("No parser set, defautling to SimpleHttpClientParser.");
            parser = new com.torunski.crawler.parser.httpclient.SimpleHttpClientParser(true);
        }
        
        // set default crawler model
        if (model == null) {
        	log.debug("No model set, defautling to MaxDepthModel.");
        	// TODO remove parameter!
            model = new com.torunski.crawler.model.MaxDepthModel(1);
        }

        // initialize stop watch
        StopWatch total = new StopWatch();
        total.start();
        
        // add at least one link to the list
        model.add(null, server + start);
        
        // starts the crawling process
        start();

        total.stop();
        
        // output some statistics
        if (log.isInfoEnabled()) {
            
            Collection visitedURIs = model.getVisitedURIs();
            Collection toVisitURIs = model.getToVisitURIs();

            log.info("Visited URIs: " + visitedURIs.size());
            
            if (toVisitURIs.size() > 0) {
                log.warn("still URIs to be visited, at least: " + toVisitURIs.size());
            }
            
            // output stop watch data
            log.info("Total time: " + total.getTime() + " ms");
        }
    }

	/**
     * Starts the crawling process in a multi threaded enviroment.
	 * @see com.torunski.crawler.core.ICrawler#start()
	 */
	public void start() {
		// loop until there aren't any URIs anymore
        while (!isFinished()) {
        	
        	synchronized(UPDATE_LOCK) {
        		if (!model.isEmpty()) {
                    // remove a link from the stack
                    Link link = model.pop();
                    loadService.execute(new LoadTask(link));
        		}
        	}
        	
        	try {
        		if (model.isEmpty())
        			Thread.currentThread().sleep(100);
			} catch (InterruptedException e) {
				log.info("Sleep of " + this.getClass().getName() + " interrupted", e);
			}
            
        }
        
        loadService.shutdown();
		parseService.shutdown();
	}
    
    private boolean isFinished() {
    	synchronized(UPDATE_LOCK) {
    		// FIXME find a better way to check if there aren't any threads left
        	return model.isEmpty() && (threadsToBeFinished == 0);
    	}
    }
    
    private class LoadTask implements Runnable {
    	
    	private final Link link;
    	
    	public LoadTask(Link link) {
			threadsToBeFinished++;
			this.link = link;
			log.debug("LoadTask created for link: " + link.getURI());
    	}

		public void run() {
			log.debug("LoadTask running for link: " + link.getURI());

			try {
				// load the page of the link
	            PageData pageData = parser.load(link);
	            
	            if (pageData.getStatus() == PageData.OK) {
	            	// no sync needed because we are still running
	                parseService.execute(new ParseTask(link, pageData));
	            }
			} finally {
				threadsToBeFinished--;
			}

			log.debug("LoadTask finished for link: " + link.getURI());
		}
    	
    }
    
    private class ParseTask implements Runnable {

    	private final Link link;
    	private final PageData pageData;
    	
    	public ParseTask(Link link, PageData pageData) {
			threadsToBeFinished++;
    		this.link = link;
			this.pageData = pageData;
			log.debug("ParseTask created for link: " + link.getURI());
    	}

		public void run() {
			log.debug("ParseTask running for link: " + link.getURI());
			
			try {
				// get the links in the page
	            Collection newURIs = parser.parse(pageData, linkFilter);

	            // update model
	            synchronized(UPDATE_LOCK) {
	                fireParserEvent(link, pageData, newURIs);
	                
	                // remove already visited URIs from the new URI list
	                newURIs.removeAll(model.getVisitedURIs());
	                
	                // the rest of the URIs can be visited
	                model.add(link, newURIs);
	            }
			} finally {
				threadsToBeFinished--;
			}
			
			log.debug("ParseTask finished for link: " + link.getURI());
		}
    	
    }


}
