/*
 * Copyright 2005 by Lars Torunski
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.torunski.crawler.events;

import java.util.Collection;

import com.torunski.crawler.core.ICrawler;
import com.torunski.crawler.link.Link;
import com.torunski.crawler.parser.PageData;

/**
 * Parsing event for the listeners.
 * 
 * @author Lars Torunski
 * @version $Revision: 1.4 $
 */
public class ParserEvent {

    /** The crawler that can be attached to the event to specify the event's origin. */
    protected ICrawler crawler = null;
    
    /** The link of the page data */
    protected Link link = null;
    
    /** The page data content of the event */
    protected PageData pageData = null;

    /** The outgoing links which are filted by the crawler filter */
    protected Collection outLinks = null;

    /**
     * No-argument constructor so subtypes can easily implement <code>Serializable</code>
     */
    public ParserEvent() {
    }

    /**
     * Creates a parser event object that came from the specified origin.
     *
     * @param crawler a crawler that indicates where this event was fired from.
     * This value is optional; <code>null</code> can be passed in if the
     * crawler origin is not required.
     * @param link the link of the PageData
     * @param pageData the page data content of this event.
     * @param outLinks outgoing links
     */
    public ParserEvent(ICrawler crawler, Link link, PageData pageData, Collection outLinks) {
        this.crawler = crawler;
        this.link = link;
        this.pageData = pageData;
        this.outLinks = outLinks;
    }

    /**
     * Retrieves the crawler origin of this event, if one was specified. This is most
     * useful when an event handler causes another event to fire - by checking
     * the origin the handler is able to prevent recursive events being
     * fired.
     * @return returns the origin of this event.
     */
    public ICrawler getCrawler() {
        return crawler;
    }
    
    /**
     * @return Returns the link.
     */
    public Link getLink() {
        return link;
    }
    
    /**
     * @return retuns the page data content object of this event.
     */
    public PageData getPageData() {
        return pageData;
    }
   
    /**
     * @return Returns the outLinks.
     */
    public Collection getOutLinks() {
        return outLinks;
    }
}
