/*
 * Copyright 2005 by Lars Torunski
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.torunski.crawler.link;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.TreeSet;

/**
 * A graph link contains all incoming and outgoing links.
 *
 * @author Lars Torunski
 * @version $Revision: 1.3 $
 */
public class LinkGraph extends Link {
    
    /**
     * @param uri the URI of the link
     */
    public LinkGraph(String uri) {
        super(uri);
    }

    private static final LinkComparator LINK_COMPARATOR = new LinkComparator();

    // --- Fujaba ---

    /**
     * <pre>
     *            *       links       * 
     * LinkGraph ----------------------- LinkGraph
     *            outLinks      inLinks 
     * </pre>
     */
    private Collection inLinks;

    public boolean addToInLinks(LinkGraph value) {
        boolean changed = false;
        if (value != null) {
            if (this.inLinks == null) {
                this.inLinks = new TreeSet(LINK_COMPARATOR);
            }

            changed = this.inLinks.add(value);
            if (changed) {
                value.addToOutLinks(this);
            }

        }
        return changed;
    }

    public boolean hasInInLinks(LinkGraph value) {
        return ((this.inLinks != null) && (value != null) && this.inLinks.contains(value));
    }

    public Iterator iteratorOfInLinks() {
        return ((this.inLinks == null) ? Collections.EMPTY_SET.iterator() : this.inLinks.iterator());
    }

    public void removeAllFromInLinks() {
        LinkGraph tmpValue;
        Iterator iter = this.iteratorOfInLinks();
        while (iter.hasNext()) {
            tmpValue = (LinkGraph) iter.next();
            this.removeFromInLinks(tmpValue);
        }
    }

    public boolean removeFromInLinks(LinkGraph value) {
        boolean changed = false;
        if ((this.inLinks != null) && (value != null)) {

            changed = this.inLinks.remove(value);
            if (changed) {
                value.removeFromOutLinks(this);
            }

        }
        return changed;
    }

    public int sizeOfInLinks() {
        return ((this.inLinks == null) ? 0 : this.inLinks.size());
    }

    /**
     * <pre>
     *            *       links       * 
     * LinkGraph ----------------------- LinkGraph
     *            inLinks      outLinks 
     * </pre>
     */
    private Collection outLinks;

    public boolean addToOutLinks(LinkGraph value) {
        boolean changed = false;
        if (value != null) {
            if (this.outLinks == null) {
                this.outLinks = new TreeSet(LINK_COMPARATOR);
            }

            changed = this.outLinks.add(value);
            if (changed) {
                value.addToInLinks(this);
            }

        }
        return changed;
    }

    public boolean hasInOutLinks(LinkGraph value) {
        return ((this.outLinks != null) && (value != null) && this.outLinks.contains(value));
    }

    public Iterator iteratorOfOutLinks() {
        return ((this.outLinks == null) ? Collections.EMPTY_SET.iterator() : this.outLinks.iterator());
    }

    public void removeAllFromOutLinks() {
        LinkGraph tmpValue;
        Iterator iter = this.iteratorOfOutLinks();
        while (iter.hasNext()) {
            tmpValue = (LinkGraph) iter.next();
            this.removeFromOutLinks(tmpValue);
        }
    }

    public boolean removeFromOutLinks(LinkGraph value) {
        boolean changed = false;
        if ((this.outLinks != null) && (value != null)) {

            changed = this.outLinks.remove(value);
            if (changed) {
                value.removeFromInLinks(this);
            }

        }
        return changed;
    }

    public int sizeOfOutLinks() {
        return ((this.outLinks == null) ? 0 : this.outLinks.size());
    }

    public void removeYou() {
        removeAllFromOutLinks();
        removeAllFromInLinks();
    }

}
