/*
 * Copyright 2005 by Lars Torunski
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.torunski.crawler.lucene;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.lucene.demo.HTMLDocument;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexWriter;

import com.torunski.crawler.events.IParserEventListener;
import com.torunski.crawler.events.ParserEvent;
import com.torunski.crawler.filter.ILinkFilter;
import com.torunski.crawler.parser.PageData;

/**
 * LuceneParserEventListener for indexing the HTML documents.
 * 
 * @author Lars Torunski
 * @version $Revision: 1.4 $
 */
public class LuceneParserEventListener implements IParserEventListener {
    
    private static transient final Log log = LogFactory.getLog(LuceneParserEventListener.class);

    /** index writer to built the index */
    private IndexWriter writer;
    
    /** separate filter for the indexed files */ 
    private ILinkFilter filter = new LuceneLinkFilter();
    
    /**
     * @param writer the Lucene index writer
     */
    public LuceneParserEventListener(IndexWriter writer) {
        this.writer = writer;
    }
    
    /**
     * Writing the content to a temporary file and indexing the content.  
     * 
     * @see com.torunski.crawler.events.IParserEventListener#parse(ParserEvent)
     */
    public void parse(ParserEvent event) {
        PageData pageData = event.getPageData();
        String path = pageData.getLink().getURI();
        
        if ((filter == null) || (filter.accept(path, path))) {
            File file = null;
            try {
                // create temp file
                file = File.createTempFile("crawler", null);
                log.debug("Temp file " + file);
                try {
                    // write data to temp file
                    Object data = pageData.getData();
                    FileWriter writer = new FileWriter(file);
                    try {
                        // string representation of the index
                    	writer.write(data.toString());
                    } finally {
                    	writer.close();
                    }
                    // index temp file
                    indexDocs(file, pageData.getLink().getURI());
                } finally {
                    file.delete();
                }
            } catch (IOException e) {
                log.error("Error creating or deleting temporary file: " + file, e);
            }
        }
        
    }

    /**
     * @return returns the filter for the to be index links.
     */
    public ILinkFilter getFilter() {
        return filter;
    }
    
    /**
     * @param filter Sets the filter for the to be indexed links.
     */
    public void setFilter(ILinkFilter filter) {
        this.filter = filter;
    }

    /**
     * Indexing the html file with the link.
     * @param file the file to be indexed.
     * @param link the link of the html page.
     */
    private void indexDocs(File file, String link) {
        try {
            // parse HTML document
            Document doc = HTMLDocument.Document(file);
            // reset url doc field
            doc.removeField("url");
            doc.add(new Field("url", link, Field.Store.YES, Field.Index.NO));
            // add docs unconditionally
            writer.addDocument(doc);
        } catch (IOException e) {
            log.error("IOException indexing temporary file", e);
        } catch (InterruptedException e) {
            log.error("InterruptedException indexing temporary file", e);
        } 
    }

}
