/*
 * Copyright 2005 by Lars Torunski
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.torunski.crawler.model;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.TreeSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.torunski.crawler.link.Link;
import com.torunski.crawler.link.LinkDepth;
import com.torunski.crawler.link.LinkDepthComparator;

/**
 * Simple graph model parser without registrating in and outcoming links.
 * It stops to a maximum depth.
 *
 * @author Lars Torunski
 * @version $Revision: 1.8 $
 */
public class MaxDepthModel implements ICrawlerModel {

    private static transient final Log log = LogFactory.getLog(MaxDepthModel.class);

    /** The default depth */
    public static final int DEFAULT_MAX_DEPTH = 4;
    
    /** The max depth */
    private int depth;
    
    /** A map of the visited links */
    private HashMap visitedURIs = new HashMap();
    
    /** A set of the to be visited links */
    private TreeSet toVisitURIs = new TreeSet(new LinkDepthComparator());
    
    /** A find links fast */
    private HashMap toVisitURIsFind = new HashMap();

    /**
     * Constructor for Crawler.
     */
    public MaxDepthModel() {
        this(DEFAULT_MAX_DEPTH);        
    }
    
    /**
     * Constructor for Crawler.
     * @param depth the max depth.
     */
    public MaxDepthModel(int depth) {
        this.depth = depth;

        log.debug("Crawler model: " + MaxDepthModel.class.getName());
        log.debug("- max depth=" + depth);
    }
 
    /**
     * @see com.torunski.crawler.model.ICrawlerModel#isEmpty()
     */
    public boolean isEmpty() {
        // check if there is at least one link left
        if (toVisitURIs.size() == 0) {
            return true;
        }
        
        // get the next element (first element in the set)
        LinkDepth l = (LinkDepth) toVisitURIs.first();
        
        return l.getDepth() > depth;
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#pop()
     */
    public synchronized Link pop() {
        // check constraint
        if (toVisitURIs.size() == 0) {
            throw new NoSuchElementException("No more URIs in MaxDepthModel.");
        }
        
        // get the next element and remove it from the list
        LinkDepth l = (LinkDepth) toVisitURIs.first();
        toVisitURIs.remove(l);

        // check constraint
        if (l.getDepth() > depth) {
            throw new NoSuchElementException("Max depth reached in MaxDepthModel.");
        }
        
        // mark this link as visited
        visitedURIs.put(l.getURI(), l);
        
        // return the link
        return l;
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#add(com.torunski.crawler.link.Link, java.lang.String)
     */
    public void add(Link origin, String uri) {
        int originDepth = getDepth(origin);
        addInternal(originDepth, uri);
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#add(com.torunski.crawler.link.Link, java.util.Collection)
     */
    public void add(Link origin, Collection uri) {
        int originDepth = getDepth(origin);
        
        Iterator iter = uri.iterator();
        while (iter.hasNext()) {
            addInternal(originDepth, (String) iter.next());
        }
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#getVisitedURIs()
     */
    public Collection getVisitedURIs() {
        return visitedURIs.values();
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#getToVisitURIs()
     */
    public Collection getToVisitURIs() {
        return toVisitURIs;
    }
    
    // --- internal methods and/or classes ---
    
    /**
     * @param Link the URI of the link
     * @return returns the depth of a visited uri.
     */
    private int getDepth(Link link) {
        if (link == null) {
            return -1;
        }
        
        LinkDepth l = (LinkDepth) visitedURIs.get(link.getURI());
        
        if (l != null) {
            return l.getDepth();
        } else {
            // this is the root
            return -1;
        }
    }

    /**
     * Adds a new URI to the to be visted list.
     * @param originDepth the depth of the origin URI
     * @param uri the link of the new URI
     */
    private synchronized void addInternal(int originDepth, String uri) {
        // find the link via the hashcode
        LinkDepth l = (LinkDepth) toVisitURIsFind.get(uri);
        
        // the depth of the uri
        int depth = originDepth + 1;
        
        // is the link new
        if (l == null) {
            l = (LinkDepth) createLink(null, uri, depth);
            toVisitURIsFind.put(uri, l);
            toVisitURIs.add(l);
        } else {
            // check if depth is to change
            if (depth < l.getDepth())  {
                toVisitURIs.remove(l);
                l.setDepth(depth);
                toVisitURIs.add(l);
            }
        }
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#createLink(java.lang.String, java.lang.String)
     */
    public Link createLink(String originUri, String uri) {
        return createLink(originUri, uri, 0);
    }
    
    /**
     * Creates a <b>new</b> link based on the orginUri and uri parameter.
     * @param originUri the origin URI of the link
     * @param uri the new URI
     * @param depth the depth of the link
     */
    public LinkDepth createLink(String originUri, String uri, int depth) {
        return new LinkDepth(uri, depth);
    }

}
