/*
 * Copyright 2005 by Lars Torunski
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.torunski.crawler.model;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.torunski.crawler.link.Link;

/**
 * Fast model with a small memory footprint and cpu usage which stops if a number of pages are parsed.
 *
 * @author Lars Torunski
 * @version $Revision: 1.6 $
 */
public class MaxIterationsModel implements ICrawlerModel {

    private static transient final Log log = LogFactory.getLog(MaxIterationsModel.class);

    /** The default number of iterations */
    public static final int DEFAULT_MAX_ITERATIONS = 32;
    
    /** The max iterations */
    private int iterations;
    
    /** A map of the visited links */
    private HashMap visitedURIs = new HashMap();
    
    /** A map of the missed visited links */
    private HashMap toVisitURIs = new HashMap();

    /**
     * Constructor for Crawler.
     */
    public MaxIterationsModel() {
        this(DEFAULT_MAX_ITERATIONS);        
    }
    
    /**
     * Constructor for Crawler.
     * @param iterations the max iterations.
     */
    public MaxIterationsModel(int iterations) {
        this.iterations = iterations;

        log.debug("Crawler model: " + MaxIterationsModel.class.getName());
        log.debug("- max iterations=" + iterations);
    }
 
    /**
     * @see com.torunski.crawler.model.ICrawlerModel#isEmpty()
     */
    public boolean isEmpty() {
        return (toVisitURIs.size() == 0) || (iterations <= 0);
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#pop()
     */
    public synchronized Link pop() {
        // reduce the iterations without a check
        iterations--;
        
        // remove a link from the stack
        Link link = (Link) toVisitURIs.values().iterator().next();
        toVisitURIs.remove(link.getURI());
        
        // mark this link as visited
        visitedURIs.put(link.getURI(), link);
        
        // return the URI
        return link;
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#add(com.torunski.crawler.link.Link, java.lang.String)
     */
    public void add(Link origin, String uri) {
        // in this crawler type we can ignore the originUri
        addInternal(uri);
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#add(com.torunski.crawler.link.Link, java.util.Collection)
     */
    public void add(Link origin, Collection uri) {
        // in this crawler type we can ignore the originUri
        // the rest of the links can be visited (but avoid double entries via the HashSet)
        Iterator iter = uri.iterator();
        while (iter.hasNext()) {
            addInternal((String) iter.next());
        }
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#getVisitedURIs()
     */
    public Collection getVisitedURIs() {
        return visitedURIs.values();
    }

    /**
     * @see com.torunski.crawler.model.ICrawlerModel#getToVisitURIs()
     */
    public Collection getToVisitURIs() {
        return toVisitURIs.values();
    }

    // --- internal methods and/or classes ---
    
    /**
     * Adds a new URI to the to be visted list without checking the visited URIs.
     * @param uri the link of the new URI
     */
    private synchronized void addInternal(String uri) {
        // find the link via the hashcode
        Link l = (Link) findLinkMap.get(uri);
        
        // is the link new
        if (l == null) {
            l = createLink(null, uri);
            toVisitURIs.put(uri, l);
            findLinkMap.put(uri, l);
        }
    }

    private HashMap findLinkMap = new HashMap();
    
    /**
     * @see com.torunski.crawler.model.ICrawlerModel#createLink(java.lang.String, java.lang.String)
     */
    public Link createLink(String orginUri, String uri) {
        // avoid duplicated links
        Link link = (Link) findLinkMap.get(uri);

        // is the link new
        if (link == null) {
            link = new Link(uri);
            findLinkMap.put(uri, link);
        }
        
        return link;
    }

}
