/*
 * Copyright 2006 by Lars Torunski
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.torunski.crawler.parser.filesystem;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.torunski.crawler.filter.ILinkFilter;
import com.torunski.crawler.link.Link;
import com.torunski.crawler.parser.IParser;
import com.torunski.crawler.parser.PageData;
import com.torunski.crawler.util.LinksUtil;

/**
 * A file system parser.
 * 
 * @author Lars Torunski
 * @version $Revision: 1.2 $
 */
public class FileSystemParser implements IParser {

    private static transient final Log log = LogFactory.getLog(FileSystemParser.class);
    
    private boolean warnOfMissingMappings = false;
    
    private HashMap mapping = new HashMap();

    public FileSystemParser() {
    }
    
    public FileSystemParser(String domain, File file) {
    	this();
    	addMapping(domain, file);
    }
    
    /**
     * Adds a mapping of a domain to a local file path
     * @param domain the domain name of the local file path
     * @param file the local file path of the domain 
     */
    public void addMapping(String domain, File file) {
    	// remove all / from end of the domain
    	while (domain.endsWith("/")) {
    		domain = domain.substring(0, domain.length() - 1);
    	}
    	mapping.put(domain, file);
    }

	/**
	 * @return true if missing mappings are reported. 
	 */
	public boolean isWarnOfMissingMappings() {
		return warnOfMissingMappings;
	}

	/**
	 * @param warnOfMissingMappings true for reporting missing mappings of domain file system maps.
	 */
	public void setWarnOfMissingMappings(boolean warnOfMissingMappings) {
		this.warnOfMissingMappings = warnOfMissingMappings;
	}
	
	// FIXME performance extract the server of the uri and use domain
    private String getDomain(String uri) {
    	Iterator itr = mapping.keySet().iterator();
    	while (itr.hasNext()) {
    		String domain = (String) itr.next();
    		if (uri.startsWith(domain)) {
    			return domain;
    		}
    	}
		return null;
	}

    /**
     * @see com.torunski.crawler.parser.IParser#load(com.torunski.crawler.link.Link)
     */
    public PageData load(Link link) {
    	final String uri = link.getURI();
    	
    	String domain = getDomain(uri);
    	if (domain != null) {

    		File file = new File((File) mapping.get(domain), uri.substring(domain.length()));
			if (file.exists()) {
				log.debug("Loading file=" + file);

				StringBuffer content = new StringBuffer(8192);
				char[] buffer = new char[4096];
				int read = 0;
				
				BufferedReader in = null; 
				try {
					in = new BufferedReader(new FileReader(file));
					while ((read = in.read(buffer)) > 0) {
						content.append(buffer, 0, read);
					}
				} catch (Exception e) {
	    			log.warn("Can't read file=" + file, e);
					return new PageDataFileSystem(link, PageData.ERROR);
			    } finally {
					if (in != null) {
						try {
							in.close();
						} catch (Exception ignore) {
						}
					}
				}
				return new PageDataFileSystem(link, content.toString());
			} else {
    			log.warn("File doesn't exists, link=" + link.getURI() + ", file=" + file);
            }
    	} else {
    		if (warnOfMissingMappings) {
    			log.warn("Missing domain file path mapping for " + uri);
    		} else {
    			log.info("Missing domain file path mapping for " + uri);
    		}
    	}
    	
    	return new PageDataFileSystem(link, PageData.ERROR);
    }

	/**
	 * @see com.torunski.crawler.parser.IParser#parse(com.torunski.crawler.parser.PageData,
	 *      com.torunski.crawler.filter.ILinkFilter)
	 */
    public Collection parse(PageData pageData, ILinkFilter linkFilter) {
    	if (!(pageData instanceof PageDataFileSystem)) {
    		log.warn("Type mismatch in " + this.getClass().getName());
    		return Collections.EMPTY_LIST;
    	}
        return LinksUtil.retrieveLinks(pageData.getLink().getURI(), (String) pageData.getData(), linkFilter);
    }
    
    // --- PageData implementation ---

    private static class PageDataFileSystem extends PageData {

        /** the data of the page */
        private String data;
        
        /**
         * @param uri the uri of the data
         * @param data the data of the uri
         */
        public PageDataFileSystem(Link link, String data) {
            super(link, PageData.OK);
            this.data = data;
        }

        /**
         * @param uri the uri of the data
         * @param status the status
         */
        public PageDataFileSystem(Link link, int status) {
            super(link, status);
        }

        /**
         * @see com.torunski.crawler.parser.PageData#getData()
         */
        public Object getData() {
            return data;
        }
        
    }

}
