/*
 * Copyright 2006 by Lars Torunski
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.torunski.crawler.parser.httpclient;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.methods.GetMethod;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Allows easy downloading of ressources from the internet
 * to a specified filesystem.
 * 
 * @author Lars Torunski
 * @version $Revision: 1.1 $
 */
public class DownloadHelper {

    private static transient final Log log = LogFactory.getLog(DownloadHelper.class);

    /** the mapping of destination and origin of URIs */
    private Map mapping;

    /** the http client */
    private HttpClient client = null;
    
    /**
     * Constructor for DownloadHelper for single http connections.
     * @param mapping a map of URIs parts as the key to the file path destination.
     */
    public DownloadHelper(Map mapping) {
    	this(mapping, false);
    }

  	/**
     * Constructor for DownloadHelper. Should be used, if a single instance is used in a multi threaded enviroment.
     * @param mapping a map of URIs parts as the key to the file path destination.
     * @param multiThreaded true for creating a multi threaded connection manager else only a single connection is allowed
     */
    public DownloadHelper(Map mapping, boolean multiThreaded) {
    	this.mapping = new HashMap(mapping.size());

    	Iterator itr = mapping.keySet().iterator();
        while (itr.hasNext()) {
        	String key = (String) itr.next();
        	String path = (String) mapping.get(key);
        	if (this.mapping.put(key.toLowerCase(), path) != null) {
        		log.info("DownloadHelper: double URI path for key=" + key);
        	}
        }
        
    	if (multiThreaded) {
            // Creates one instance of HttpClient with a MultiThreadedHttpConnectionManager.
            client = new HttpClient(new MultiThreadedHttpConnectionManager());
    	} else {
    		// Creates one instance of HttpClient with a single connection manager.
            client = new HttpClient();
    	}
    }

    /**
     * @param uri the URI the be downloaded
     * @param filename the file name of the destination
     * @return true if the URI was downloaded to the filesystem
     */
    public boolean download(String uri, String filename) {
    	log.info("downloading URI='" + uri + "' into file='" + filename + "'");
    	
    	File file = new File(filename);
        // check if file exists before testing if parent exists
        if (!file.exists()) {
            // check if the directory structure required exists and create it if it doesn't
            File filepath = new File(file.getParent());
            if (!filepath.exists()) {
                filepath.mkdirs();
            }
        }
        
        // Create a method instance.
        GetMethod httpGet = null;
        try {
            httpGet = new GetMethod(uri);
        } catch (Exception e) {
            log.info("Download failed '" + httpGet.getStatusLine() + "' for URI=" + uri);
            return false;
        }
       
        try {
          // Execute the method.
          int statusCode = client.executeMethod(httpGet);

          if (statusCode != HttpStatus.SC_OK) {
              log.info("Download failed '" + httpGet.getStatusLine() + "' for URI=" + uri);
              httpGet.releaseConnection();
          	  return false;
          }

          // read the response body as a stream
          Reader reader = new InputStreamReader(httpGet.getResponseBodyAsStream(), httpGet.getResponseCharSet());
          try {
        	  BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), httpGet.getResponseCharSet()));
        	  try {
                  char[] buffer = new char[4096];
                  int len;
                  while ((len = reader.read(buffer)) > 0) {
                	  writer.write(buffer, 0, len);
                  }
              } finally {
            	  writer.close();
              }
          } finally {
        	  reader.close();
          }
        } catch (IOException e) {
            log.warn("Failed to download URI='" + uri + "' with charset='" + httpGet.getResponseCharSet() + "'", e);
        } finally {
          // Release the connection.
          httpGet.releaseConnection();
        }
        
    	return true;
    }

    /**
     * Downloads a uri to a filesystem which is mapped by the mapping list.
     * @param uri the URI to be downloaded
     * @return true if the uri was downloaded to the filesystem
     */
    public boolean download(String uri) {
    	String dest = getDestination(uri);
    	if (dest != null) {
    		return download(uri, dest);
    	} else {
        	log.warn("DownloadHelper: no file destination found for URI=" + uri);
        	return false;
    	}
    }
    
    /**
     * Returns the file path and name of the in the parameter specified URI.
     * @param uri the URI of the ressource
     * @return the file path and name of the URI's destination, returns null if no matching can be found.
     */
    public String getDestination(String uri) {
    	if ((uri == null) || (mapping == null)) return null;
    	
    	String find = uri.toLowerCase();
    	
    	Iterator itr = mapping.keySet().iterator();
        while (itr.hasNext()) {
        	String uriPath = (String) itr.next();
        	
        	if (find.startsWith(uriPath)) {
    			return (String) mapping.get(uriPath) + uri.substring(uriPath.length());
        	}
        }
        return null;
    }

}
