/*
 * Copyright 2005 by Lars Torunski
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.torunski.crawler.parser.httpclient;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Collection;
import java.util.Collections;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.commons.httpclient.Credentials;
import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HeaderElement;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.ProxyHost;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;

import com.torunski.crawler.filter.ILinkFilter;
import com.torunski.crawler.link.Link;
import com.torunski.crawler.parser.IParser;
import com.torunski.crawler.parser.PageData;
import com.torunski.crawler.util.LinksUtil;


/**
 * A simple parser based on Apache Jakarta Commons HttpClient to show how to
 * implement a own parser.
 * 
 * This parse provides full support for HTTP over Secure Sockets Layer (SSL) or
 * IETF Transport Layer Security (TLS) protocols by leveraging the Java Secure
 * Socket Extension (JSSE).
 * 
 * http://jakarta.apache.org/commons/httpclient/
 * 
 * @author Lars Torunski
 * @version $Revision: 1.11 $
 */
public class SimpleHttpClientParser implements IParser {

    private static transient final Log log = LogFactory.getLog(SimpleHttpClientParser.class);

    /** the http client */
    private HttpClient client = null;
    
    /**
     * The constructor of SimpleHttpClientParser for single http connections.
     */
    public SimpleHttpClientParser() {
    	this(false);
    }
    
    /**
     * Creates an instance of SimpleHttpClientParser.
     * @param multiThreaded true for creating a multi threaded connection manager else only a single connection is allowed
     */
    public SimpleHttpClientParser(boolean multiThreaded) {
    	if (multiThreaded) {
            // Creates one instance of HttpClient with a MultiThreadedHttpConnectionManager.
            client = new HttpClient(new MultiThreadedHttpConnectionManager());
    	} else {
    		// Creates one instance of HttpClient with a single connection manager.
            client = new HttpClient();
    	}
    }
    
    /**
     * Set the proxy settings.
     * @param proxyHost The proxy host
     * @param proxyPort The proxy port
     */
    public void setProxy(String proxyHost, int proxyPort) {
        client.getHostConfiguration().setProxyHost(new ProxyHost(proxyHost, proxyPort));
    }

    /**
     * Returns the proxyHost.
     * @return the proxy host, or <code>null</code> if not set
     */
    public String getProxyHost() {
    	return client.getHostConfiguration().getProxyHost();
    }

    /**
     * Returns the proxyPort.
     * @return the proxy port, or <code>-1</code> if not set
     */
    public int getProxyPort() {
    	return client.getHostConfiguration().getProxyPort();
    }
    
    /** 
     * Sets the proxy credentials for the given authentication realm. 
     * Any previous credentials for the given realm will be overwritten.
     * 
     * @param authscope the authentication scope
     * @param credentials the authentication credentials for the given realm.
     * 
     * @see org.apache.commons.httpclient.HttpState#setProxyCredentials(AuthScope, Credentials) 
     */
    public void setProxyCredentials(AuthScope authscope, Credentials credentials) {
        client.getState().setProxyCredentials(authscope, credentials);
    }
    
    /**
     * Get the proxy credentials for the given authentication scope.
     *
     * @param authscope the {@link AuthScope authentication scope}
     * @return the credentials 
     * 
     * @see org.apache.commons.httpclient.HttpState#getProxyCredentials(AuthScope) 
     */
    public Credentials getProxyCredentials(AuthScope authscope) {
    	return client.getState().getProxyCredentials(authscope);
    }

    /** 
     * Sets the credentials for the given authentication scope. 
     * Any previous credentials for the given scope will be overwritten.
     * 
     * @param authscope the authentication scope
     * @param credentials the authentication credentials for the given scope.
     * 
     * @see org.apache.commons.httpclient.HttpState#setCredentials(AuthScope, Credentials) 
     */
    public void setCredentials(AuthScope authscope, Credentials credentials) {
        client.getState().setCredentials(authscope, credentials);
    }
    
    /**
     * Get the credentials for the given authentication scope.
     *
     * @param authscope the authentication scope
     * @return the credentials 
     * 
     * @see org.apache.commons.httpclient.HttpState#getCredentials(AuthScope) 
     */
    public Credentials getCredentials(AuthScope authscope) {
    	return client.getState().getCredentials(authscope);
    }
    
    // --- implementation of the interface methods ---
    
    /**
     * @see com.torunski.crawler.parser.IParser#load(com.torunski.crawler.link.Link)
     */
    public PageData load(Link link) {
        String uri = link.getURI();
        log.info("download: " + uri);

        // Create a method instance.
        GetMethod httpGet = null;
        try {
            httpGet = new GetMethod(uri);
        } catch (Exception e) {
            log.info("Method failed: " + httpGet.getStatusLine() + " for " + uri);
            return new PageDataHttpClient(link, PageData.ERROR);
        }
       
        // Provide a custom retry handler
        httpGet.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, new DefaultHttpMethodRetryHandler(5, false));

        StringBuffer responseBody = new StringBuffer(64 * 1024);
        try {
          // Execute the method.
          int statusCode = client.executeMethod(httpGet);

          if (statusCode != HttpStatus.SC_OK) {
            log.info("Method failed: " + httpGet.getStatusLine() + " for " + uri);
          }

          if (!containsText(httpGet)) {
        	  log.warn("URL does not contain text: " + uri);
        	  responseBody = null;
          } else {
              // read the response body as a stream
              Reader reader = new InputStreamReader(httpGet.getResponseBodyAsStream(), httpGet.getResponseCharSet());
              char[] buffer = new char[4096];
              int len;
              while ((len = reader.read(buffer)) > 0) {
            	  responseBody.append(buffer, 0, len);
              }
          }
        } catch (IOException e) {
            log.warn("Failed to download file = " + uri, e);
        } finally {
          // Release the connection.
          httpGet.releaseConnection();
        }
        
        if (responseBody != null) {
            return new PageDataHttpClient(link, responseBody.toString());
        } else {
            return new PageDataHttpClient(link, PageData.ERROR);
        }
    }

    private boolean containsText(GetMethod method) {
        Header contentType = method.getResponseHeader("content-type");
        if (contentType != null) {
        	HeaderElement[] elements = contentType.getElements();
        	for (int i=0; i < elements.length; i++) {
                String name = elements[i].getName();
        		if ((name != null) && (name.startsWith("text")))
        			return true;
        	}
        	// if no correct content-type is found, so it isn't text
        	return false;
        }
    	// if no content type is set, it may be text
		return true;
	}

	/**
     * @see com.torunski.crawler.parser.IParser#parse(com.torunski.crawler.parser.PageData, com.torunski.crawler.filter.ILinkFilter)
     */
    public Collection parse(PageData pageData, ILinkFilter linkFilter) {
    	if (!(pageData instanceof PageDataHttpClient)) {
    		log.warn("Type mismatch in " + this.getClass().getName());
    		return Collections.EMPTY_LIST;
    	}
        return LinksUtil.retrieveLinks(pageData.getLink().getURI(), (String) pageData.getData(), linkFilter);
    }
    
    // --- PageData implementation ---

    private static class PageDataHttpClient extends PageData {

        /** the data of the page */
        private String data;
        
        /**
         * @param uri the uri of the data
         * @param data the data of the uri
         */
        public PageDataHttpClient(Link link, String data) {
            super(link, PageData.OK);
            this.data = data;
        }

        /**
         * @param uri the uri of the data
         * @param status the status
         */
        public PageDataHttpClient(Link link, int status) {
            super(link, status);
        }

        /**
         * @see com.torunski.crawler.parser.PageData#getData()
         */
        public Object getData() {
            return data;
        }
        
    }

}
